// set the dimensions and margins of the graph
const width = 800;
const height = 500;

// append the svg object to the body of the page
const svg = d3
  .select('#my_dataviz')
  .append('svg')
  .attr('width', width / 2)
  .attr('height', height);

const svg2 = d3
  .select('#my_dataviz')
  .append('svg')
  .attr('width', width / 2)
  .attr('height', height);

function render(mapData, countryData) {
  var projection = d3
    .geoEqualEarth()
    .fitSize([width / 2, height], mapData);
  var path = d3.geoPath(projection);

  const mapColorAttr = '2018';
  var mapColorScale = d3
    .scaleLinear()
    .domain(
      d3.extent(
        d3.map(
          countryData.values(),
          (r) => +r[mapColorAttr],
        ),
      ),
    )
    .range(['red', 'blue']);

  function getCountryColor(name) {
    let entry = countryData.get(name);
    if (!(entry == undefined)) {
      return mapColorScale(entry[mapColorAttr]);
    }
    return 'grey';
  }

  svg
    .append('g')
    .selectAll('path')
    .data(mapData.features)
    .join('path')
    // draw each country
    .attr('fill', function (d) {
      return getCountryColor(d.properties.name);
    })
    .on('mouseover', function (event, data) {
      //d3.select(this).attr('fill', 'red');
      d3.selectAll(`#${this.id}`)
        .attr('fill', 'red')
        .attr('r', 20);
    })
    .on('mouseleave', function (event, data) {
      d3.selectAll(`#${this.id}`)
        .attr('fill', getCountryColor(data.properties.name))
        .attr('r', (d) => rScale(d[rAttr]));
    })
    .attr('id', (d) => d.properties.name)
    .attr('d', path);

  const xAttr = 'Name';
  const yAttr = '2018';
  const barWidth = 7;

  const x = d3
    .scaleBand()
    .domain(countryData.keys())
    .range([0, width / 2]);

  const y = d3
    .scaleLinear()
    .domain([
      0,
      d3.max(
        d3.map(
          countryData.values().filter((d) => +d[yAttr] > 0),
          (d) => +d[yAttr],
        ),
      ),
    ])
    .nice()
    .range([0, height - 50]);

  svg2
    .append('g')
    .attr('transform', `translate(0,${height - 50})`) // This controls the vertical position of the Axis
    .call(d3.axisBottom(x));

  svg2
    .append('g')
    .attr('transform', `translate(0,0)`) // This controls the vertical position of the Axis
    .call(d3.axisRight(y));

  svg2
    .append('g')
    .selectAll('rect')
    .data(
      countryData
        .values()
        .filter((c) => +c[yAttr] > 200000000000),
    )
    .join('rect')
    .attr('x', (d) => x(d[xAttr]) - barWidth / 2)
    .attr('y', (d) => y(d[yAttr]))
    .attr('width', barWidth)
    .attr('height', (d) => height - y(d[yAttr]))
    .attr('id', (d) => d.Name)
    .on('mouseover', function (event, data) {
      console.log(data);
      d3.select(this).attr('fill', 'red');
    })
    .on('mouseleave', function (event, data) {
      d3.select(this).attr(
        'fill',
        getCountryColor(data.Name),
      );
    });
}

//Read the data
const mapUrl =
  'https://raw.githubusercontent.com/holtzy/D3-graph-gallery/master/DATA/world.geojson';
const countryStatsUrl =
  'https://raw.githubusercontent.com/thehankstah/cp341/main/Military%20Spending%201960-2019.csv';
const continentUrl =
  'https://raw.githubusercontent.com/samayo/country-json/master/src/country-by-continent.json';

function normalizeCountryName(s) {
  return s.replaceAll(' ', '_').toLowerCase();
}

Promise.all([
  d3.json(mapUrl),
  d3.csv(countryStatsUrl),
  d3.json(continentUrl),
]).then(([mapData, csvData, continentData]) => {
  var countryData = new Map();
  csvData.forEach((row) => {
    row.Name = normalizeCountryName(row.Name);
    countryData.set(row.Name, row);
  });
  mapData.features.forEach((f) => {
    f.properties.name = normalizeCountryName(
      f.properties.name,
    );
  });
  console.log(countryData);
  render(mapData, countryData);
});
